// WideMargin. Simple fast bible software.
// Copyright (C) 2011  Daniel Hughes
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;

namespace WideMargin.Database
{
	public class HtmlTemplate
	{
		Dictionary<string, object> _parameters = new Dictionary<string, object>();
		
		internal void AddParameter<T>(string name, T parameterValue)
		{
			_parameters.Add (name, parameterValue);
		}
		
		string GetListDefinition(string template, int startIndex)
		{
			var stopIndex = template.IndexOf('>', startIndex + 1);
			return template.Substring(startIndex, stopIndex-startIndex + 1);
		}
		
		string GetListContents(string template, string start, string end)
		{
			int startIndex = template.IndexOf(start) + start.Length;
			int endIndex = template.IndexOf(end);
			return template.Substring(startIndex,endIndex - startIndex);
		}
		
		string ReflectionReplace(string template, object obj, string objectName)
		{
			while(true)
			{
				int startIndex = template.IndexOf (string.Format ("<#={0}.", objectName));
				if(startIndex < 0)
				{
					return template;	
				}
				//find property name
				int endIndex = template.IndexOf(">", startIndex) + 1;
				string fullTag = template.Substring(startIndex, endIndex - startIndex);
				string propertyName = fullTag.Split('.')[1].Replace("#>", string.Empty);
				
				string val = obj.GetType().GetProperty(propertyName).GetValue(obj, null).ToString();
				template = template.Replace(fullTag, val);
			}
		}
		
		internal string Generate(string template)
		{
			//sort out the lists
			int index = template.IndexOf("<#List");
			if(index >= 0)
			{
				var listDefinition = GetListDefinition(template, index);
				var listDefinitionParts = listDefinition.Replace("#>",string.Empty).Split(',');
				var listName = listDefinitionParts[1];
				var individualName = listDefinitionParts[2];
				
				var listContents = GetListContents(template, listDefinition, "<#EndList," + listName + "#>");
				var list = ((IEnumerable)_parameters[listName]).Cast<object>();
				var listBuilder = new StringBuilder();
				foreach(var listElement in list)
				{
					string listContent = listContents.Replace(string.Format ("<#={0}#>", individualName), listElement.ToString ());
					listContent = ReflectionReplace(listContent, listElement, individualName);
					listBuilder.Append(listContent);
				}
				template = template.Replace(listDefinition + listContents + "<#EndList," + listName + "#>", listBuilder.ToString());
			}
			
			foreach(string name in _parameters.Keys)
			{
				template = template.Replace(string.Format ("<#={0}#>", name), _parameters[name].ToString ());
			}
			return template;
		}
	}
}

