// WideMargin. Simple fast bible software.
// Copyright (C) 2013  Daniel Hughes
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;
using SharpSword;
using System.Collections.Generic;

namespace WideMargin.Database
{
	public class VerseWithBoldTerms : IVerse
	{
		public VerseWithBoldTerms (IVerse verse, IEnumerable<string> searchTerms)
		{
			Identifier = verse.Identifier;
			
			string text = verse.Text;
			foreach(var searchTerm in searchTerms)
			{
				text = MakeTermBold(searchTerm, text);
			}
			
			Text = text;
		}

		public IVerseIdentifier Identifier 
		{
			get;
			private set;
		}

		public string Text 
		{
			get ;
			private set;
		}
		
		string MakeTermBold(string term, string text)
		{
			int startIndex = 0;
			
			//handle text that starts with the search term
			if(text.StartsWith(term, StringComparison.CurrentCultureIgnoreCase) && 
			   !char.IsLetter(text[term.Length]))
			{
				text = MakeBold(text, 0, term.Length, out startIndex);	
			}
			
			//handle term with a space before and after it (we don't want partial word matches)
			int indexOfTerm = text.IndexOf(term, startIndex, StringComparison.CurrentCultureIgnoreCase);
			while(indexOfTerm != -1)
			{
				if(IsFullTermAtIndex(text, term, indexOfTerm))
				{
					text = MakeBold(text, indexOfTerm, term.Length, out startIndex);					
                }
				else
				{
					//this is partal match need to advance the startindex to prevent getting it again
					startIndex++;
				}
				indexOfTerm = text.IndexOf(term, startIndex, StringComparison.CurrentCultureIgnoreCase);
			}
			return text;
		}
		
		bool IsFullTermAtIndex(string text, string term, int index)
		{
			if(!char.IsLetter(text[index - 1]) && 
			   index + term.Length < text.Length &&
			   !char.IsLetter(text[index + term.Length]))
			{
				return true; //full term in middle of text		
			}
			if(!char.IsLetter(text[index - 1]) && 
			   index + term.Length == text.Length)
			{
				return true; //full term at end of text;
			}
			return false;
		}
		
		string MakeBold(string text, int start, int length, out int end)
		{
			text = text.Insert (start + length,"</b>");
			text = text.Insert (start, "<b>");
			end = start + length + 7;
			return text;
		}
	}
}

