﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WideMargin.MVCInterfaces;
using WideMargin.Utilities;

namespace WideMargin.Gui.Wpf
{
    /// <summary>
    /// Interaction logic for FilteredView.xaml
    /// </summary>
    public partial class FilteredView : UserControl, IFilteredView
    {
        public FilteredView()
        {
            InitializeComponent();
            _webView.RequestVersification += WebView_RequestVersification;
            _webView.VerseLinkClicked += WebView_VerseLinkClicked;
            _webView.ActionLinkClicked += WebView_ActionLinkClicked;
            _webView.RequestMore += WebView_RequestMore;
        }

        void WebView_RequestMore(object sender, EventArgs e)
        {
            MoreResultsRequired.Fire(this, e);
        }

        void WebView_ActionLinkClicked(object sender, LinkClickedEventArgs<string> e)
        {
            ActionLinkClicked.Fire(this, e);
        }

        void WebView_VerseLinkClicked(object sender, LinkClickedEventArgs<SharpSword.IVerseIdentifier> e)
        {
            VerseLinkClicked.Fire(this, e);
        }

        void WebView_RequestVersification(object sender, RequestVersificationEventArgs e)
        {
            RequestVersification.Fire(this, e);
        }

        IEnumerable<Filter> _filters;

        public IEnumerable<Filter> Filters
        {
            get
            {
                return _filters;
            }
            set
            {
                _filters = value;
                this.Dispatcher.Invoke((Action)(() =>
                {
                    _buttons.Children.Clear();
                    foreach (var filter in value)
                    {
                        var button = new Button() 
                        { 
                            Content = filter.Name ,
                            Margin=new Thickness(5,1,5,1),
                            FontSize = (double)new FontSizeConverter().ConvertFromString("11pt")
                        };
                        _buttons.Children.Add(button);

                        System.Action view = filter.PerformFilter;
                        button.Click += (sender, args) => view();
                    }
                }));
            }
        }

        public void SetResults(string resultsHtml, IEnumerable<Filter> filters)
        {
            Filters = filters;
            Dispatcher.Invoke((Action)(() =>
            {
                _webView.Text = resultsHtml;
            }));
        }

        public event EventHandler<EventArgs> MoreResultsRequired;

        public void AppendText(string text)
        {
            Dispatcher.Invoke((Action)(() =>
            {
                _webView.AppendText(text);
            }));
        }

        public string Text
        {
            set 
            {
                Dispatcher.Invoke((Action)(() =>
                {
                    _webView.Text = value;
                }));
            }
        }

        public event EventHandler<LinkClickedEventArgs<SharpSword.IVerseIdentifier>> VerseLinkClicked;

        public event EventHandler<LinkClickedEventArgs<string>> ActionLinkClicked;

        public event EventHandler<RequestVersificationEventArgs> RequestVersification;
    }
}
